@@ -86,14 +86,18 @@ local rest       = token("default",   any)
 local comment    = token("comment",   P("%") * (1-S("\n\r"))^0)
 local internal   = token("reserved",  exact_match(mergedshortcuts,false))
 local shortcut   = token("data",      exact_match(mergedinternals))
+
 local helper     = token("command",   exact_match(metafuncommands))
 local plain      = token("plain",     exact_match(metapostcommands))
 local quoted     = token("quote",     dquote)
                  * token("string",    P(1-dquote)^0)
                  * token("quote",     dquote)
-local texstuff   = token("quote",     P("btex ") + P("verbatimtex "))
-                 * token("string",    P(1-P(" etex"))^0)
-                 * token("quote",     P(" etex"))
+local separator  = P(" ") + S("\n\r")^1
+local btex       = (P("btex") + P("verbatimtex")) * separator
+local etex       = separator * P("etex")
+local texstuff   = token("quote",     btex)
+                 * token("string",    (1-etex)^0)
+                 * token("quote",     etex)
 local primitive  = token("primitive", exact_match(metapostprimitives))
 local identifier = token("default",   cstoken^1)
 local number     = token("number",    number)
@@ -125,15 +129,18 @@ lexer.embed_lexer(metafunlexer, cldlexer, startluacode, stopluacode)
 
 local luacall      = token("embedded",P("lua") * ( P(".") * R("az","AZ","__")^1 )^1)
 
+local keyword      = token("default", (R("AZ","az","__")^1) * # P(space^0 * P("=")))
+
 metafunlexer._rules = {
     { "whitespace", spacing    },
     { "comment",    comment    },
+    { "keyword",    keyword    },  -- experiment, maybe to simple
     { "internal",   internal   },
     { "shortcut",   shortcut   },
+    { "luacall",    luacall    },
     { "helper",     helper     },
     { "plain",      plain      },
     { "primitive",  primitive  },
-    { "luacall",    luacall    },
     { "texstuff",   texstuff   },
     { "suffix",     suffix     },
     { "identifier", identifier },
