@@ -14,10 +14,9 @@ local suffix, addsuffix, removesuffix, replacesuffix = file.suffix, file.addsuff
 local nameonly, basename, joinpath, collapsepath = file.nameonly, file.basename, file.join, file.collapsepath
 local lower = string.lower
 local concat = table.concat
-local write_nl = texio.write_nl
+local write_nl = (logs and logs.writer) or (texio and texio.write_nl) or print
 
-local otfversion  = 2.826
-local otlversion  = 3.029
+local otlversion  = 3.113
 
 local helpinfo = [[
 <?xml version="1.0"?>
@@ -25,13 +24,12 @@ local helpinfo = [[
  <metadata>
   <entry name="name">mtx-fonts</entry>
   <entry name="detail">ConTeXt Font Database Management</entry>
-  <entry name="version">0.21</entry>
+  <entry name="version">1.00</entry>
  </metadata>
  <flags>
   <category name="basic">
    <subcategory>
-    <flag name="save"><short>save open type font in raw table (ff format)</short></flag>
-    <flag name="convert"><short>save open type font in raw table (ctx format)</short></flag>
+    <flag name="convert"><short>save open type font in raw table</short></flag>
     <flag name="unpack"><short>save a tma file in a more readable format</short></flag>
    </subcategory>
    <subcategory>
@@ -80,7 +78,6 @@ local helpinfo = [[
     <example><command>mtxrun --script font --list --file --pattern=*somename*</command></example>
    </subcategory>
    <subcategory>
-    <example><command>mtxrun --script font --save texgyrepagella-regular.otf</command></example>
     <example><command>mtxrun --script font --convert texgyrepagella-regular.otf</command></example>
     <example><command>mtxrun --script font --convert --names texgyrepagella-regular.otf</command></example>
    </subcategory>
@@ -108,14 +105,20 @@ local function loadmodule(filename)
     end
 end
 
--- old loader code
-
-loadmodule("font-otp.lua") -- we need to unpack the font for analysis
-
--- new loader code
+-- loader code
 
 loadmodule("char-def.lua")
 
+loadmodule("font-ini.lua")
+loadmodule("font-log.lua")
+loadmodule("font-con.lua")
+loadmodule("font-cft.lua")
+loadmodule("font-enc.lua")
+loadmodule("font-agl.lua")
+loadmodule("font-cid.lua")
+loadmodule("font-map.lua")
+loadmodule("font-oti.lua")
+
 loadmodule("font-otr.lua")
 loadmodule("font-cff.lua")
 loadmodule("font-ttf.lua")
@@ -123,6 +126,7 @@ loadmodule("font-tmp.lua")
 loadmodule("font-dsp.lua")
 loadmodule("font-oup.lua")
 
+loadmodule("font-otl.lua")
 loadmodule("font-onr.lua")
 
 -- extra code
@@ -264,7 +268,7 @@ local function showfeatures(tag,specification)
         report()
         indeed("instances : % t",instancenames)
     end
-    local features = fonts.helpers.getfeatures(specification.filename,not getargument("nosave"))
+    local features, tables = fonts.helpers.getfeatures(specification.filename,not getargument("nosave"))
     if features then
         for what, v in table.sortedhash(features) do
             local data = features[what]
@@ -272,7 +276,7 @@ local function showfeatures(tag,specification)
                 report()
                 report("%s features:",what)
                 report()
-                report("feature  script   languages")
+                report("  feature  script   languages")
                 report()
                 for f,ff in table.sortedhash(data) do
                     local done = false
@@ -284,7 +288,7 @@ local function showfeatures(tag,specification)
                         else
                             done = true
                         end
-                        report("% -8s % -8s % -8s",f,s,concat(table.sortedkeys(ss), " ")) -- todo: padd 4
+                        report("  % -8s % -8s % -8s",f,s,concat(table.sortedkeys(ss), " ")) -- todo: padd 4
                     end
                 end
             end
@@ -292,7 +296,26 @@ local function showfeatures(tag,specification)
     else
         report("no features")
     end
+    if tables then
+        tables = table.tohash(tables)
+        local methods = {
+            overlay = (tables.colr or tables.cpal) and { format = "cff/ttf", feature = "color:overlay" } or nil,
+            bitmap  = (tables.cblc or tables.cbdt) and { format = "png",     feature = "color:bitmap"  } or nil,
+            outline = (tables.svg                ) and { format = "svg",     feature = "color:svg"     } or nil,
+        }
+        if next(methods) then
+            report()
+            report("color features:")
+            report()
+            report("  method   feature         formats")
+            report()
+            for k, v in table.sortedhash(methods) do
+                report("  % -8s % -14s  %s",k,v.feature,v.format)
+            end
+        end
+    end
     report()
+    collectgarbage("collect")
 end
 
 local function reloadbase(reload)
@@ -345,6 +368,7 @@ local function list_matches(t,info)
             for k=1,#s do
                 local v = s[k]
                 showfeatures(v,t[v])
+                collectgarbage("collect") -- in case we load a lot
             end
         else
             for k=1,#s do
@@ -419,7 +443,7 @@ function scripts.fonts.list()
         end
     elseif pattern then
         --~ mtxrun --script font --list --pattern=*somename*
-       list_matches(fonts.names.list(string.topattern(pattern,true),reload,all),info)
+        list_matches(fonts.names.list(string.topattern(pattern,true),reload,all),info)
     elseif given then
         --~ mtxrun --script font --list somename
         list_matches(fonts.names.list(given,reload,all),info)
@@ -432,18 +456,6 @@ function scripts.fonts.list()
 
 end
 
-function scripts.fonts.justload()
-    local fullname = environment.files[1]
-    if fullname then
-        local result = fontloader.open(fullname)
-        if type(result) == "table" then
-            report("loading %s: %s","succeeded",fullname)
-            fontloader.close(result)
-        end
-    end
-    report("loading %s: %s","failed",fullname)
-end
-
 function scripts.fonts.unpack()
     local name = removesuffix(basename(givenfiles[1] or ""))
     if name and name ~= "" then
@@ -456,8 +468,6 @@ function scripts.fonts.unpack()
             report("fontsave, saving data in %s",savename)
             if data.creator == "context mkiv" then
                 fonts.handlers.otf.readers.unpack(data)
-            else
-                fonts.handlers.otf.unpackdata(data)
             end
             io.savedata(savename,table.serialize(data,true))
         else
@@ -466,51 +476,6 @@ function scripts.fonts.unpack()
     end
 end
 
-function scripts.fonts.save()
-    local name = givenfiles[1] or ""
-    local sub  = givenfiles[2] or ""
-    local function save(savename,fontblob)
-        if fontblob then
-            if fontblob.validation_state and table.contains(fontblob.validation_state,"bad_ps_fontname") then
-                report("ignoring bad fontname for %a",name)
-                savename = nameonly(name) .. "-bad-ps-name"
-            end
-            savename = addsuffix(lower(savename),"lua")
-            report("fontsave, saving data in %a",savename)
-            table.tofile(savename,fontloader.to_table(fontblob),"return")
-            fontloader.close(fontblob)
-        end
-    end
-    if name and name ~= "" then
-        local filename = resolvers.findfile(name) -- maybe also search for opentype
-        if filename and filename ~= "" then
-            local suffix = lower(suffix(filename))
-            if suffix == 'ttf' or suffix == 'otf' or suffix == 'ttc' or suffix == "dfont" then
-                local fontinfo = fontloader.info(filename)
-                if fontinfo then
-                    report("font: %a located as %a",name,filename)
-                    if #fontinfo > 0 then
-                        for k=1,#fontinfo do
-                            local v = fontinfo[k]
-                            save(v.fontname,fontloader.open(filename,v.fullname))
-                        end
-                    else
-                        save(fontinfo.fullname,fontloader.open(filename))
-                    end
-                else
-                    report("font: %a cannot be read",filename)
-                end
-            else
-                report("font: %a not saved",filename)
-            end
-        else
-            report("font: %a not found",name)
-        end
-    else
-        report("font: no name given")
-    end
-end
-
 function scripts.fonts.convert() -- new save
     local name = givenfiles[1] or ""
     local sub  = givenfiles[2] or ""
@@ -521,11 +486,16 @@ function scripts.fonts.convert() -- new save
             if suffix == 'ttf' or suffix == 'otf' or suffix == 'ttc' then
                 local data = fonts.handlers.otf.readers.loadfont(filename,sub)
                 if data then
+                    local nofsubfonts = data and data.properties and data.properties.nofsubfonts or 0
                     fonts.handlers.otf.readers.compact(data)
                     fonts.handlers.otf.readers.rehash(data,getargument("names") and "names" or "unicodes")
                     local savename = replacesuffix(lower(data.metadata.fullname or filename),"lua")
                     table.save(savename,data)
-                    report("font: %a saved as %a",filename,savename)
+                    if nofsubfonts == 0 then
+                        report("font: %a saved as %a",filename,savename)
+                    else
+                        report("font: %a saved as %a, %i subfonts found, provide number if wanted",filename,savename,nofsubfonts)
+                    end
                 else
                     report("font: %a not loaded",filename)
                 end
@@ -550,12 +520,8 @@ if getargument("list") then
     scripts.fonts.list()
 elseif getargument("reload") then
     scripts.fonts.reload()
-elseif getargument("save") then
-    scripts.fonts.save()
 elseif getargument("convert") then
     scripts.fonts.convert()
-elseif getargument("justload") then
-    scripts.fonts.justload()
 elseif getargument("unpack") then
     scripts.fonts.unpack()
 elseif getargument("statistics") then
