@@ -192,7 +192,6 @@ insert(characters.ranges,{
     end,
 })
 
-
 local blocks = allocate {
     ["adlam"]                                      = { first = 0x1E900, last = 0x1E95F,             description = "Adlam" },
     ["aegeannumbers"]                              = { first = 0x10100, last = 0x1013F,             description = "Aegean Numbers" },
@@ -236,6 +235,7 @@ local blocks = allocate {
     ["cherokee"]                                   = { first = 0x013A0, last = 0x013FF, otf="cher", description = "Cherokee" },
     ["cherokeesupplement"]                         = { first = 0x0AB70, last = 0x0ABBF,             description = "Cherokee Supplement" },
     ["chesssymbols"]                               = { first = 0x1FA00, last = 0x1FA6F,             description = "Chess Symbols" },
+    ["chorasmian"]                                 = { first = 0x10FB0, last = 0x10FDF,             description = "Chorasmian" },
     ["cjkcompatibility"]                           = { first = 0x03300, last = 0x033FF, otf="hang", description = "CJK Compatibility" },
     ["cjkcompatibilityforms"]                      = { first = 0x0FE30, last = 0x0FE4F, otf="hang", description = "CJK Compatibility Forms" },
     ["cjkcompatibilityideographs"]                 = { first = 0x0F900, last = 0x0FAFF, otf="hang", description = "CJK Compatibility Ideographs" },
@@ -250,6 +250,7 @@ local blocks = allocate {
     ["cjkunifiedideographsextensiond"]             = { first = 0x2B740, last = 0x2B81F,             description = "CJK Unified Ideographs Extension D" },
     ["cjkunifiedideographsextensione"]             = { first = 0x2B820, last = 0x2CEAF,             description = "CJK Unified Ideographs Extension E" },
     ["cjkunifiedideographsextensionf"]             = { first = 0x2CEB0, last = 0x2EBEF,             description = "CJK Unified Ideographs Extension F" },
+    ["cjkunifiedideographsextensiong"]             = { first = 0x30000, last = 0x3134F,             description = "CJK Unified Ideographs Extension G" },
     ["combiningdiacriticalmarks"]                  = { first = 0x00300, last = 0x0036F,             description = "Combining Diacritical Marks" },
     ["combiningdiacriticalmarksextended"]          = { first = 0x01AB0, last = 0x01AFF,             description = "Combining Diacritical Marks Extended" },
     ["combiningdiacriticalmarksforsymbols"]        = { first = 0x020D0, last = 0x020FF,             description = "Combining Diacritical Marks for Symbols" },
@@ -298,6 +299,7 @@ local blocks = allocate {
  -- ["digitsthai"]                                 = { first = 0x00E50, last = 0x00E59, math = true },
  -- ["digitstibetan"]                              = { first = 0x00F20, last = 0x00F29, math = true },
     ["dingbats"]                                   = { first = 0x02700, last = 0x027BF,              description = "Dingbats" },
+    ["divesakuru"]                                 = { first = 0x11900, last = 0x1195F,              description = "Dives Akuru" },
     ["dogra"]                                      = { first = 0x11800, last = 0x1184F,              description = "Dogra" },
     ["dominotiles"]                                = { first = 0x1F030, last = 0x1F09F,              description = "Domino Tiles" },
     ["duployan"]                                   = { first = 0x1BC00, last = 0x1BC9F,              description = "Duployan" },
@@ -361,6 +363,7 @@ local blocks = allocate {
     ["katakanaphoneticextensions"]                 = { first = 0x031F0, last = 0x031FF, otf="kana",  description = "Katakana Phonetic Extensions" },
     ["kayahli"]                                    = { first = 0x0A900, last = 0x0A92F,              description = "Kayah Li" },
     ["kharoshthi"]                                 = { first = 0x10A00, last = 0x10A5F, otf="khar",  description = "Kharoshthi" },
+    ["khitansmallscript"]                          = { first = 0x18B00, last = 0x18CFF,              description = "Khitan Small Script" },
     ["khmer"]                                      = { first = 0x01780, last = 0x017FF, otf="khmr",  description = "Khmer" },
     ["khmersymbols"]                               = { first = 0x019E0, last = 0x019FF, otf="khmr",  description = "Khmer Symbols" },
     ["khojki"]                                     = { first = 0x11200, last = 0x1124F,              description = "Khojki" },
@@ -380,6 +383,7 @@ local blocks = allocate {
     ["linearbideograms"]                           = { first = 0x10080, last = 0x100FF, otf="linb",  description = "Linear B Ideograms" },
     ["linearbsyllabary"]                           = { first = 0x10000, last = 0x1007F, otf="linb",  description = "Linear B Syllabary" },
     ["lisu"]                                       = { first = 0x0A4D0, last = 0x0A4FF,              description = "Lisu" },
+    ["lisusupplement"]                             = { first = 0x11FB0, last = 0x11FBF,              description = "Lisu Supplement" },
     ["lowercasebold"]                              = { first = 0x1D41A, last = 0x1D433, math = true },
     ["lowercaseboldfraktur"]                       = { first = 0x1D586, last = 0x1D59F, math = true },
     ["lowercasebolditalic"]                        = { first = 0x1D482, last = 0x1D49B, math = true },
@@ -504,6 +508,7 @@ local blocks = allocate {
     ["suttonsignwriting"]                          = { first = 0x1D800, last = 0x1DAAF,              description = "Sutton SignWriting" },
     ["sylotinagri"]                                = { first = 0x0A800, last = 0x0A82F, otf="sylo",  description = "Syloti Nagri" },
     ["symbolsandpictographsextendeda"]             = { first = 0x1FA70, last = 0x1FAFF,              description = "Symbols and Pictographs Extended-A" },
+    ["symbolsforlegacycomputing"]                  = { first = 0x1FB00, last = 0x1FBFF,              description = "Symbols for Legacy Computing" },
     ["syriac"]                                     = { first = 0x00700, last = 0x0074F, otf="syrc",  description = "Syriac" },
     ["syriacsupplement"]                           = { first = 0x00860, last = 0x0086F,              description = "Syriac Supplement" },
     ["tagalog"]                                    = { first = 0x01700, last = 0x0171F, otf="tglg",  description = "Tagalog" },
@@ -517,6 +522,7 @@ local blocks = allocate {
     ["tamil"]                                      = { first = 0x00B80, last = 0x00BFF, otf="taml",  description = "Tamil" },
     ["tamilsupplement"]                            = { first = 0x11FC0, last = 0x11FFF,              description = "Tamil Supplement" },
     ["tangut"]                                     = { first = 0x17000, last = 0x187FF,              description = "Tangut" },
+    ["tangutsupplement"]                           = { first = 0x18D00, last = 0x18D8F,              description = "Tangut Supplement" },
     ["tangutcomponents"]                           = { first = 0x18800, last = 0x18AFF,              description = "Tangut Components" },
     ["telugu"]                                     = { first = 0x00C00, last = 0x00C7F, otf="telu",  description = "Telugu" },
     ["thaana"]                                     = { first = 0x00780, last = 0x007BF, otf="thaa",  description = "Thaana" },
@@ -555,6 +561,7 @@ local blocks = allocate {
     ["verticalforms"]                              = { first = 0x0FE10, last = 0x0FE1F,              description = "Vertical Forms" },
     ["wancho"]                                     = { first = 0x1E2C0, last = 0x1E2FF,              description = "Wancho" },
     ["warangciti"]                                 = { first = 0x118A0, last = 0x118FF,              description = "Warang Citi" },
+    ["yezidi"]                                     = { first = 0x10E80, last = 0x10EBF,              description = "Yezidi" },
     ["yijinghexagramsymbols"]                      = { first = 0x04DC0, last = 0x04DFF, otf="yi",    description = "Yijing Hexagram Symbols" },
     ["yiradicals"]                                 = { first = 0x0A490, last = 0x0A4CF, otf="yi",    description = "Yi Radicals" },
     ["yisyllables"]                                = { first = 0x0A000, last = 0x0A48F, otf="yi",    description = "Yi Syllables" },
@@ -755,11 +762,11 @@ local is_spacing = allocate ( tohash {
 } )
 
 local is_mark = allocate ( tohash {
-    "mn", "ms",
+    "mn", "ms", -- "mn", "mc",
 } )
 
 local is_punctuation = allocate ( tohash {
-    "pc","pd","ps","pe","pi","pf","po",
+    "pc", "pd", "ps", "pe", "pi", "pf", "po",
 } )
 
 local is_symbol = allocate ( tohash {
