@@ -45,7 +45,6 @@ specifiers.variants = variants
 definers.methods    = definers.methods or { }
 
 local internalized  = allocate() -- internal tex numbers (private)
-local lastdefined   = nil -- we don't want this one to end up in s-tra-02
 
 local loadedfonts   = constructors.loadedfonts
 local designsizes   = constructors.designsizes
@@ -80,53 +79,6 @@ and prepares a table that will move along as we proceed.</p>
 -- name name(sub) name(sub)*spec name*spec
 -- name@spec*oeps
 
-local splitter, splitspecifiers = nil, "" -- not so nice
-
-local P, C, S, Cc = lpeg.P, lpeg.C, lpeg.S, lpeg.Cc
-
-local left  = P("(")
-local right = P(")")
-local colon = P(":")
-local space = P(" ")
-
-definers.defaultlookup = "file"
-
-local prefixpattern = P(false)
-
-local function addspecifier(symbol)
-    splitspecifiers     = splitspecifiers .. symbol
-    local method        = S(splitspecifiers)
-    local lookup        = C(prefixpattern) * colon
-    local sub           = left * C(P(1-left-right-method)^1) * right
-    local specification = C(method) * C(P(1)^1)
-    local name          = C((1-sub-specification)^1)
-    splitter = P((lookup + Cc("")) * name * (sub + Cc("")) * (specification + Cc("")))
-end
-
-local function addlookup(str,default)
-    prefixpattern = prefixpattern + P(str)
-end
-
-definers.addlookup = addlookup
-
-addlookup("file")
-addlookup("name")
-addlookup("spec")
-
-local function getspecification(str)
-    return lpegmatch(splitter,str or "") -- weird catch
-end
-
-definers.getspecification = getspecification
-
-function definers.registersplit(symbol,action,verbosename)
-    addspecifier(symbol)
-    variants[symbol] = action
-    if verbosename then
-        variants[verbosename] = action
-    end
-end
-
 local function makespecification(specification,lookup,name,sub,method,detail,size)
     size = size or 655360
     if not lookup or lookup == "" then
@@ -151,13 +103,65 @@ local function makespecification(specification,lookup,name,sub,method,detail,siz
     return t
 end
 
-
 definers.makespecification = makespecification
 
-function definers.analyze(specification, size)
-    -- can be optimized with locals
-    local lookup, name, sub, method, detail = getspecification(specification or "")
-    return makespecification(specification, lookup, name, sub, method, detail, size)
+if context then
+
+    local splitter, splitspecifiers = nil, "" -- not so nice
+
+    local P, C, S, Cc, Cs = lpeg.P, lpeg.C, lpeg.S, lpeg.Cc, lpeg.Cs
+
+    local left   = P("(")
+    local right  = P(")")
+    local colon  = P(":")
+    local space  = P(" ")
+    local lbrace = P("{")
+    local rbrace = P("}")
+
+    definers.defaultlookup = "file"
+
+    local prefixpattern = P(false)
+
+    local function addspecifier(symbol)
+        splitspecifiers     = splitspecifiers .. symbol
+        local method        = S(splitspecifiers)
+        local lookup        = C(prefixpattern) * colon
+        local sub           = left * C(P(1-left-right-method)^1) * right
+        local specification = C(method) * C(P(1)^1)
+        local name          = Cs((lbrace/"") * (1-rbrace)^1 * (rbrace/"") + (1-sub-specification)^1)
+        splitter = P((lookup + Cc("")) * name * (sub + Cc("")) * (specification + Cc("")))
+    end
+
+    local function addlookup(str)
+        prefixpattern = prefixpattern + P(str)
+    end
+
+    definers.addlookup = addlookup
+
+    addlookup("file")
+    addlookup("name")
+    addlookup("spec")
+
+    local function getspecification(str)
+        return lpegmatch(splitter,str or "") -- weird catch
+    end
+
+    definers.getspecification = getspecification
+
+    function definers.registersplit(symbol,action,verbosename)
+        addspecifier(symbol)
+        variants[symbol] = action
+        if verbosename then
+            variants[verbosename] = action
+        end
+    end
+
+    function definers.analyze(specification, size)
+        -- can be optimized with locals
+        local lookup, name, sub, method, detail = getspecification(specification or "")
+        return makespecification(specification, lookup, name, sub, method, detail, size)
+    end
+
 end
 
 --[[ldx--
@@ -203,9 +207,6 @@ function resolvers.name(specification)
                     features.normal = normal
                 end
                 normal.instance = instance
-                if not callbacks.supported.glyph_stream_provider then
-                    normal.variableshapes = true -- for the moment
-                end
             end
             --
             local suffix = lower(suffixonly(resolved))
@@ -335,7 +336,7 @@ local function checkfeatures(tfmdata)
                             for script, languages in next, scripts do
                                 if languages["*"] then
                                     -- ok
-                                elseif not languages[usedlanguage] then
+                                elseif context and not languages[usedlanguage] then
                                     report_defining("font %!font:name!, feature %a, script %a, no language %a",
                                         tfmdata,feature,script,usedlanguage)
                                 end
@@ -355,7 +356,7 @@ local function checkfeatures(tfmdata)
                                 if not languages["*"] then
                                     for i=1,#foundlanguages do
                                         local language = foundlanguages[i]
-                                        if not languages[language] then
+                                        if context and not languages[language] then
                                             report_defining("font %!font:name!, feature %a, script %a, no language %a",
                                                 tfmdata,feature,script,language)
                                         end
@@ -377,6 +378,7 @@ function definers.loadfont(specification)
     -- todo: also hash by instance / factors
     local tfmdata = loadedfonts[hash] -- hashes by size !
     if not tfmdata then
+        -- normally context will not end up here often (if so there is an issue somewhere)
         local forced = specification.forced or ""
         if forced ~= "" then
             local reader = readers[lower(forced)] -- normally forced is already lowered
@@ -449,10 +451,6 @@ not gain much. By the way, passing id's back to in the callback was
 introduced later in the development.</p>
 --ldx]]--
 
-function definers.current() -- or maybe current
-    return lastdefined
-end
-
 function definers.registered(hash)
     local id = internalized[hash]
     return id, id and fontdata[id]
@@ -491,7 +489,9 @@ function definers.read(specification,size,id) -- id can be optional, name can al
         end
     else
         tfmdata = definers.loadfont(specification) -- can be overloaded
+-- put in properties instead
         if tfmdata then
+            tfmdata.original = specification.specification
             if trace_defining then
                 report_defining("loaded and hashed: %s",hash)
             end
@@ -505,7 +505,6 @@ function definers.read(specification,size,id) -- id can be optional, name can al
             end
         end
     end
-    lastdefined = tfmdata or id -- todo ! ! ! ! !
     if not tfmdata then -- or id?
         report_defining( "unknown font %a, loading aborted",specification.name)
     elseif trace_defining and type(tfmdata) == "table" then
@@ -527,4 +526,6 @@ end
 <p>We overload the <l n='tfm'/> reader.</p>
 --ldx]]--
 
-callbacks.register('define_font', definers.read, "definition of fonts (tfmdata preparation)")
+if not context then
+    callbacks.register('define_font', definers.read, "definition of fonts (tfmdata preparation)")
+end
