@@ -1,5 +1,6 @@
 if not modules then modules = { } end modules ['node-rul'] = {
     version   = 1.001,
+    optimize  = true,
     comment   = "companion to node-rul.mkiv",
     author    = "Hans Hagen, PRAGMA-ADE, Hasselt NL",
     copyright = "PRAGMA ADE / ConTeXt Development Team",
@@ -17,9 +18,12 @@ if not modules then modules = { } end modules ['node-rul'] = {
 -- fill s withcolor .5white ;
 -- draw boundingbox s withcolor yellow;
 
+local tonumber           = tonumber
+
+local context            = context
 local attributes         = attributes
 local nodes              = nodes
-local properties         = nodes.properties
+local properties         = nodes.properties.data
 
 local enableaction       = nodes.tasks.enableaction
 
@@ -27,25 +31,31 @@ local nuts               = nodes.nuts
 local tonode             = nuts.tonode
 local tonut              = nuts.tonut
 
-local getfield           = nuts.getfield
-local setfield           = nuts.setfield
 local setnext            = nuts.setnext
 local setprev            = nuts.setprev
 local setlink            = nuts.setlink
 local getnext            = nuts.getnext
 local getprev            = nuts.getprev
 local getid              = nuts.getid
+local getdirection       = nuts.getdirection
 local getattr            = nuts.getattr
 local setattr            = nuts.setattr
 local getfont            = nuts.getfont
 local getsubtype         = nuts.getsubtype
 local getlist            = nuts.getlist
 local setwhd             = nuts.setwhd
-local setdir             = nuts.setdir
 local setattrlist        = nuts.setattrlist
 local setshift           = nuts.setshift
 local getwidth           = nuts.getwidth
 local setwidth           = nuts.setwidth
+local setoffsets         = nuts.setoffsets
+local setfield           = nuts.setfield
+
+----- getfield           = nuts.getfield
+----- getdata            = nuts.getdata
+local getruledata        = nuts.getruledata
+
+local isglyph            = nuts.isglyph
 
 local flushlist          = nuts.flush_list
 local effective_glue     = nuts.effective_glue
@@ -53,25 +63,25 @@ local insert_node_after  = nuts.insert_after
 local insert_node_before = nuts.insert_before
 local find_tail          = nuts.tail
 local setglue            = nuts.setglue
-local traverse_id        = nuts.traverse_id
-local list_dimensions    = nuts.rangedimensions
+local getrangedimensions = nuts.rangedimensions
 local hpack_nodes        = nuts.hpack
-local current_attr       = nuts.current_attr
+local copy_list          = nuts.copy_list
+
+local nexthlist          = nuts.traversers.hlist
 
 local nodecodes          = nodes.nodecodes
 local rulecodes          = nodes.rulecodes
 local gluecodes          = nodes.gluecodes
 local listcodes          = nodes.listcodes
-local kerncodes          = nodes.kerncodes
 
 local glyph_code         = nodecodes.glyph
-local localpar_code      = nodecodes.localpar
+local par_code           = nodecodes.par
 local dir_code           = nodecodes.dir
 local glue_code          = nodecodes.glue
 local hlist_code         = nodecodes.hlist
 
-local indent_code        = listcodes.indent
-local line_code          = listcodes.line
+local indentlist_code    = listcodes.indent
+local linelist_code      = listcodes.line
 
 local leftskip_code      = gluecodes.leftskip
 local rightskip_code     = gluecodes.rightskip
@@ -82,6 +92,7 @@ local nodepool           = nuts.pool
 local new_rule           = nodepool.rule
 local new_userrule       = nodepool.userrule
 local new_kern           = nodepool.kern
+local new_leader         = nodepool.leader
 
 local n_tostring         = nodes.idstostring
 local n_tosequence       = nodes.tosequence
@@ -114,10 +125,13 @@ local dimenfactor        = fonts.helpers.dimenfactor
 local splitdimen         = number.splitdimen
 local setmetatableindex  = table.setmetatableindex
 
+local magicconstants     = tex.magicconstants
+local running            = magicconstants.running
+
 --
 
-local striprange         = nodes.striprange
-local processwords       = nodes.processwords
+local striprange         = nuts.striprange
+local processwords       = nuts.processwords
 
 --
 
@@ -125,51 +139,59 @@ local rules = nodes.rules or { }
 nodes.rules = rules
 rules.data  = rules.data  or { }
 
+local nutrules = nuts.rules or { }
+nuts.rules     = nutrules -- not that many
+
 storage.register("nodes/rules/data", rules.data, "nodes.rules.data")
 
 local data = rules.data
 
 -- we implement user rules here as it takes less code this way
 
-local function userrule(t,noattributes)
+local function usernutrule(t,noattributes)
     local r = new_userrule(t.width or 0,t.height or 0,t.depth or 0)
     if noattributes == false or noattributes == nil then
         -- avoid fuzzy ones
     else
-        setattrlist(r,current_attr())
+        setattrlist(r,true)
     end
     properties[r] = t
-    return tonode(r)
+    return r
 end
 
-rules.userrule    = userrule
-local ruleactions = { }
-rules.ruleactions = ruleactions
+nutrules.userrule = usernutrule
 
-local function mathradical(n,h,v)
-    ----- size    = getfield(n,"index")
-    local font    = getfield(n,"transform")
+local function userrule(t,noattributes)
+    return tonode(usernutrule(t,noattributes))
+end
+
+rules.userrule       = userrule
+local ruleactions    = { }
+
+rules   .ruleactions = ruleactions
+nutrules.ruleactions = ruleactions -- convenient
+
+local function mathaction(n,h,v,what)
+    local font    = getruledata(n)
     local actions = fontresources[font].mathruleactions
     if actions then
-        local action = actions.radicalaction
+        local action = actions[what]
         if action then
             action(n,h,v,font)
         end
     end
 end
 
+local function mathradical(n,h,v)
+    mathaction(n,h,v,"radicalaction")
+end
+
 local function mathrule(n,h,v)
-    ----- size    = getfield(n,"index")
-    local font    = getfield(n,"transform")
-    local actions = fontresources[font].mathruleactions
-    if actions then
-        local action = actions.hruleaction
-        if action then
-            action(n,h,v,font)
-        end
-    end
+    mathaction(n,h,v,"hruleaction")
 end
 
+local x
+
 local function useraction(n,h,v)
     local p = properties[n]
     if p then
@@ -189,18 +211,18 @@ local subtypeactions = {
     [rulecodes.radical]  = mathradical,
 }
 
-callbacks.register(
-    "process_rule",
-    function(n,h,v)
-        local n = tonut(n)
-        local s = getsubtype(n)
-        local a = subtypeactions[s]
-        if a then
-            a(n,h,v)
-        end
-    end,
-    "handle additional user rule features"
-)
+local function process_rule(n,h,v)
+    local n = tonut(n)
+    local s = getsubtype(n)
+    local a = subtypeactions[s]
+    if a then
+        a(n,h,v)
+    end
+end
+
+callbacks.register("process_rule",process_rule,"handle additional user rule features")
+
+callbacks.functions.process_rule = process_rule
 
 --
 
@@ -208,15 +230,26 @@ local trace_ruled   = false  trackers.register("nodes.rules", function(v) trace_
 local report_ruled  = logs.reporter("nodes","rules")
 
 function rules.define(settings)
-    data[#data+1] = settings
-    context(#data)
+    local nofdata = #data + 1
+    data[nofdata] = settings
+    local text = settings.text
+    if text then
+        local b = nuts.takebox(text)
+        if b then
+            nodepool.register(b)
+            settings.text = getlist(b)
+        else
+            settings.text = nil
+        end
+    end
+    return nofdata
 end
 
 local function flush_ruled(head,f,l,d,level,parent,strip) -- not that fast but acceptable for this purpose
     local font = nil
-    local id   = getid(f)
-    if id == glyph_code then
-        font = getfont(f)
+    local char, id = isglyph(f)
+    if char then
+        font = id
     elseif id == hlist_code then
         font = getattr(f,a_runningtext)
     end
@@ -238,11 +271,10 @@ local function flush_ruled(head,f,l,d,level,parent,strip) -- not that fast but a
     if not f then
         return head
     end
-    local w, ht, dp     = list_dimensions(parent,f,getnext(l))
+    local wd, ht, dp    = getrangedimensions(parent,f,getnext(l))
     local method        = d.method
     local empty         = d.empty == v_yes
     local offset        = d.offset
-    local continue      = d.continue
     local dy            = d.dy
     local order         = d.order
     local max           = d.max
@@ -280,7 +312,7 @@ local function flush_ruled(head,f,l,d,level,parent,strip) -- not that fast but a
         m = 0
     end
 
-    local function inject(r,w,ht,dp)
+    local function inject(r,wd,ht,dp)
         if layer then
             setattr(r,a_viewerlayer,layer)
         end
@@ -291,7 +323,7 @@ local function flush_ruled(head,f,l,d,level,parent,strip) -- not that fast but a
             setnext(l)
             flushlist(f)
         else
-            local k = new_kern(-w)
+            local k = new_kern(-wd)
             if foreground then
                 insert_node_after(head,l,k)
                 insert_node_after(head,k,r)
@@ -303,12 +335,12 @@ local function flush_ruled(head,f,l,d,level,parent,strip) -- not that fast but a
         end
         if trace_ruled then
             report_ruled("level %a, width %p, height %p, depth %p, nodes %a, text %a",
-                level,w,ht,dp,n_tostring(f,l),n_tosequence(f,l,true))
+                level,wd,ht,dp,n_tostring(f,l),n_tosequence(f,l,true))
         end
     end
     if mp and mp ~= "" then
-        local r = userrule {
-            width  = w,
+        local r = usernutrule {
+            width  = wd,
             height = ht,
             depth  = dp,
             type   = "mp",
@@ -320,29 +352,42 @@ local function flush_ruled(head,f,l,d,level,parent,strip) -- not that fast but a
             ca     = color,
             ta     = transparency,
         }
-        inject(tonut(r),w,ht,dp)
+        inject(r,wd,ht,dp)
     else
-        for i=1,level do
-            local ht =  (offset+(i-1)*dy)*e + rulethickness - m
-            local dp = -(offset+(i-1)*dy)*e + rulethickness + m
-            local r = new_rule(w,ht,dp)
-            if color then
-                setattr(r,a_colormodel,colorspace)
-                setattr(r,a_color,color)
+        local tx = d.text
+        if tx then
+            local l = copy_list(tx)
+            if d["repeat"] == v_yes then
+                l = new_leader(wd,l)
+                setattrlist(l,tx)
             end
-            if transparency then
-                setattr(r,a_transparency,transparency)
+            l = hpack_nodes(l,wd,"exactly")
+            inject(l,wd,ht,dp)
+        else
+            for i=1,level do
+                local hd = (offset+(i-1)*dy)*e - m
+--                 local ht =  hd + rulethickness - m
+--                 local dp = -hd + rulethickness + m
+                local ht =  hd + rulethickness
+                local dp = -hd + rulethickness
+                local r = new_rule(wd,ht,dp)
+                -- can be done more efficient
+                if color then
+                    setattr(r,a_colormodel,colorspace)
+                    setattr(r,a_color,color)
+                end
+                if transparency then
+                    setattr(r,a_transparency,transparency)
+                end
+                inject(r,wd,ht,dp)
             end
-            inject(r,w,ht,dp)
         end
     end
     return head
 end
 
-local process = nodes.processwords
-
 rules.handler = function(head)
-    return process(a_ruled,data,flush_ruled,head)
+    return processwords(a_ruled,data,flush_ruled,head)
 end
 
 function rules.enable()
@@ -355,16 +400,18 @@ local report_shifted = logs.reporter("nodes","shifting")
 
 local a_shifted = attributes.private('shifted')
 
-nodes.shifts      = nodes.shifts      or { }
-nodes.shifts.data = nodes.shifts.data or { }
+local shifts = nodes.shifts or { }
+nodes.shifts = shifts
+shifts.data  = shifts.data or { }
 
-storage.register("nodes/shifts/data", nodes.shifts.data, "nodes.shifts.data")
+storage.register("nodes/shifts/data", shifts.data, "nodes.shifts.data")
 
-local data = nodes.shifts.data
+local data = shifts.data
 
-function nodes.shifts.define(settings)
-    data[#data+1] = settings
-    context(#data)
+function shifts.define(settings)
+    local nofdata = #data + 1
+    data[nofdata] = settings
+    return nofdata
 end
 
 local function flush_shifted(head,first,last,data,level,parent,strip) -- not that fast but acceptable for this purpose
@@ -375,7 +422,7 @@ local function flush_shifted(head,first,last,data,level,parent,strip) -- not tha
     local next = getnext(last)
     setprev(first)
     setnext(last)
-    local width, height, depth = list_dimensions(parent,first,next)
+    local width, height, depth = getrangedimensions(parent,first,next)
     local list = hpack_nodes(first,width,"exactly") -- we can use a simple pack
     if first == head then
         head = list
@@ -395,26 +442,28 @@ local function flush_shifted(head,first,last,data,level,parent,strip) -- not tha
     return head
 end
 
-local process = nodes.processwords
+shifts.handler = function(head)
+    return processwords(a_shifted,data,flush_shifted,head)
+end
 
-nodes.shifts.handler = function(head) return process(a_shifted,data,flush_shifted,head) end
-
-function nodes.shifts.enable()
+function shifts.enable()
     enableaction("shipouts","nodes.shifts.handler")
 end
 
 -- linefillers
 
-nodes.linefillers      = nodes.linefillers      or { }
-nodes.linefillers.data = nodes.linefillers.data or { }
+local linefillers = nodes.linefillers or { }
+nodes.linefillers = linefillers
+linefillers.data  = linefillers.data or { }
 
-storage.register("nodes/linefillers/data", nodes.linefillers.data, "nodes.linefillers.data")
+storage.register("nodes/linefillers/data", linefillers.data, "nodes.linefillers.data")
 
-local data = nodes.linefillers.data
+local data = linefillers.data
 
-function nodes.linefillers.define(settings)
-    data[#data+1] = settings
-    context(#data)
+function linefillers.define(settings)
+    local nofdata = #data + 1
+    data[nofdata] = settings
+    return nofdata
 end
 
 local function linefiller(current,data,width,location)
@@ -425,7 +474,7 @@ local function linefiller(current,data,width,location)
     local ca     = data.ca
     local ta     = data.ta
     if mp and mp ~= "" then
-        return tonut(userrule {
+        return usernutrule {
             width     = width,
             height    = height,
             depth     = depth,
@@ -436,18 +485,56 @@ local function linefiller(current,data,width,location)
             ca        = ca,
             ta        = ta,
             option    = location,
-            direction = getdir(current),
-        })
+            direction = getdirection(current),
+        }
     else
-        local linefiller = new_rule(width,height,depth)
+        local rule = new_rule(width,height,depth)
         if ca then
-            setattr(linefiller,a_colorspace,ma)
-            setattr(linefiller,a_color,ca)
+            setattr(rule,a_colorspace,ma)
+            setattr(rule,a_color,ca)
         end
         if ta then
-            setattr(linefiller,a_transparency,ta)
+            setattr(rule,a_transparency,ta)
+        end
+        return rule
+    end
+end
+
+function linefillers.filler(current,data,width,height,depth)
+    if width and width > 0 then
+        local height = height or data.height or 0
+        local depth  = depth  or data.depth  or 0
+        if (height + depth) ~= 0 then
+            local mp = data.mp
+            local ma = data.ma
+            local ca = data.ca
+            local ta = data.ta
+            if mp and mp ~= "" then
+                return usernutrule {
+                    width     = width,
+                    height    = height,
+                    depth     = depth,
+                    type      = "mp",
+                    line      = data.rulethickness,
+                    data      = mp,
+                    ma        = ma,
+                    ca        = ca,
+                    ta        = ta,
+                    option    = location,
+                    direction = getdirection(current),
+                }
+            else
+                local rule = new_rule(width,height,depth)
+                if ca then
+                    setattr(rule,a_colorspace,ma)
+                    setattr(rule,a_color,ca)
+                end
+                if ta then
+                    setattr(rule,a_transparency,ta)
+                end
+                return rule
+            end
         end
-        return linefiller
     end
 end
 
@@ -461,120 +548,118 @@ local function find_attr(head,attr)
     end
 end
 
-function nodes.linefillers.handler(head)
--- local current = tonut(head) -- when we hook into the contributers
-    for current in traverse_id(hlist_code,tonut(head)) do
-        if getsubtype(current) == line_code then
-            local list = getlist(current)
-            if list then
-                -- why doesn't leftskip take the attributes
-                -- or list[linefiller] or maybe first match (maybe we need a fast helper for that)
-                local a = getattr(current,a_linefiller)
-                if a then
-                    local class = a % 1000
-                    local data  = data[class]
-                    if data then
-                        local location   = data.location
-                        local scope      = data.scope
-                        local distance   = data.distance
-                        local threshold  = data.threshold
-                        local leftlocal  = false
-                        local rightlocal = false
-                        --
-                        if scope == v_right then
-                            leftlocal = true
-                        elseif scope == v_left then
-                            rightlocal = true
-                        elseif scope == v_local then
-                            leftlocal  = true
-                            rightlocal = true
-                        end
-                        --
-                        if location == v_left or location == v_both then
-                            local lskip = nil -- leftskip
-                            local iskip = nil -- indentation
-                            local head  = list
-                            while head do
-                                local id = getid(head)
-                                if id == glue_code then
-                                    if getsubtype(head) == leftskip_code then
-                                        lskip = head
-                                    else
-                                        break
-                                    end
-                                elseif id == localpar_code or id == dir_code then
-                                    -- go on
-                                elseif id == hlist_code then
-                                    if getsubtype(head) == indent_code then
-                                        iskip = head
-                                    end
-                                    break
+function linefillers.handler(head)
+    for current, subtype in nexthlist, head do
+        if current and subtype == linelist_code then
+            -- why doesn't leftskip take the attributes
+            -- or list[linefiller] or maybe first match (maybe we need a fast helper for that)
+            local a = getattr(current,a_linefiller)
+            if a then
+                local class = a % 1000
+                local data  = data[class]
+                if data then
+                    local location   = data.location
+                    local scope      = data.scope
+                    local distance   = data.distance
+                    local threshold  = data.threshold
+                    local leftlocal  = false
+                    local rightlocal = false
+                    --
+                    if scope == v_right then
+                        leftlocal = true
+                    elseif scope == v_left then
+                        rightlocal = true
+                    elseif scope == v_local then
+                        leftlocal  = true
+                        rightlocal = true
+                    end
+                    --
+                    local list = getlist(current)
+                    --
+                    if location == v_left or location == v_both then
+                        local lskip = nil -- leftskip
+                        local iskip = nil -- indentation
+                        local head  = list
+                        while head do
+                            local id = getid(head)
+                            if id == glue_code then
+                                if getsubtype(head) == leftskip_code then
+                                    lskip = head
                                 else
                                     break
                                 end
-                                head = getnext(head)
+                            elseif id == par_code or id == dir_code then
+                                -- go on
+                            elseif id == hlist_code then
+                                if getsubtype(head) == indentlist_code then
+                                    iskip = head
+                                end
+                                break
+                            else
+                                break
                             end
-                            if head then
-                                local indentation = iskip and getwidth(iskip) or 0
-                                local leftfixed   = lskip and getwidth(lskip) or 0
-                                local lefttotal   = lskip and effective_glue(lskip,current) or 0
-                                local width = lefttotal - (leftlocal and leftfixed or 0) + indentation - distance
-                                if width > threshold then
-                                    if iskip then
-                                        setwidth(iskip,0)
+                            head = getnext(head)
+                        end
+                        if head then
+                            local indentation = iskip and getwidth(iskip) or 0
+                            local leftfixed   = lskip and getwidth(lskip) or 0
+                            local lefttotal   = lskip and effective_glue(lskip,current) or 0
+                            local width = lefttotal - (leftlocal and leftfixed or 0) + indentation - distance
+                            if width > threshold then
+                                if iskip then
+                                    setwidth(iskip,0)
+                                end
+                                if lskip then
+                                    setglue(lskip,leftlocal and getwidth(lskip) or nil)
+                                    if distance > 0 then
+                                        insert_node_after(list,lskip,new_kern(distance))
                                     end
-                                    if lskip then
-                                        setglue(lskip,leftlocal and getwidth(lskip) or nil)
-                                        if distance > 0 then
-                                            insert_node_after(list,lskip,new_kern(distance))
-                                        end
-                                        insert_node_after(list,lskip,linefiller(current,data,width,"left"))
-                                    else
-                                        insert_node_before(list,head,linefiller(current,data,width,"left"))
-                                        if distance > 0 then
-                                            insert_node_before(list,head,new_kern(distance))
-                                        end
+                                    insert_node_after(list,lskip,linefiller(current,data,width,"left"))
+                                else
+                                    insert_node_before(list,head,linefiller(current,data,width,"left"))
+                                    if distance > 0 then
+                                        insert_node_before(list,head,new_kern(distance))
                                     end
                                 end
                             end
                         end
-                        --
-                        if location == v_right or location == v_both then
-                            local pskip = nil -- parfillskip
-                            local rskip = nil -- rightskip
-                            local tail  = find_tail(list)
-                            while tail and getid(tail) == glue_code do
-                                local subtype = getsubtype(tail)
-                                if subtype == rightskip_code then
-                                    rskip = tail
-                                elseif subtype == parfillskip_code then
-                                    pskip = tail
-                                else
-                                    break
-                                end
-                                tail = getprev(tail)
+                    end
+                    --
+                    if location == v_right or location == v_both then
+                        local pskip = nil -- parfillskip
+                        local rskip = nil -- rightskip
+                        local tail  = find_tail(list)
+                        while tail and getid(tail) == glue_code do
+                            local subtype = getsubtype(tail)
+                            if subtype == rightskip_code then
+                                rskip = tail
+                            elseif subtype == parfillskip_code then
+                                pskip = tail
+                            else
+                                break
                             end
-                            if tail then
-                                local rightfixed = rskip and getwidth(rskip) or 0
-                                local righttotal = rskip and effective_glue(rskip,current) or 0
-                                local parfixed   = pskip and getwidth(pskip) or 0
-                                local partotal   = pskip and effective_glue(pskip,current) or 0
-                                local width = righttotal - (rightlocal and rightfixed or 0) + partotal - distance
-                                if width > threshold then
-                                    if pskip then
-                                        setglue(pskip)
+                            tail = getprev(tail)
+                        end
+                        if tail then
+                            local rightfixed = rskip and getwidth(rskip) or 0
+                            local righttotal = rskip and effective_glue(rskip,current) or 0
+                            local parfixed   = pskip and getwidth(pskip) or 0
+                            local partotal   = pskip and effective_glue(pskip,current) or 0
+                            local width = righttotal - (rightlocal and rightfixed or 0) + partotal - distance
+                            if width > threshold then
+                                if pskip then
+                                    setglue(pskip)
+                                end
+                                if rskip then
+                                    setglue(rskip,rightlocal and getwidth(rskip) or nil)
+                                    if distance > 0 then
+                                        insert_node_before(list,rskip,new_kern(distance))
                                     end
-                                    if rskip then
-                                        setglue(rskip,rightlocal and getwidth(rskip) or nil)
-                                        if distance > 0 then
-                                            insert_node_before(list,rskip,new_kern(distance))
-                                        end
-                                        insert_node_before(list,rskip,linefiller(current,data,width,"right"))
-                                    else
-                                        insert_node_after(list,tail,linefiller(current,data,width,"right"))
-                                        if distance > 0 then
-                                            insert_node_after(list,tail,new_kern(distance))
-                                        end
+                                    insert_node_before(list,rskip,linefiller(current,data,width,"right"))
+                                else
+                                    insert_node_after(list,tail,linefiller(current,data,width,"right"))
+                                    if distance > 0 then
+                                        insert_node_after(list,tail,new_kern(distance))
                                     end
                                 end
                             end
@@ -589,7 +674,7 @@ end
 
 local enable = false
 
-function nodes.linefillers.enable()
+function linefillers.enable()
     if not enable then
     -- we could now nil it
         enableaction("finalizers","nodes.linefillers.handler")
@@ -617,6 +702,8 @@ implement {
             { "ta", "integer" },
             { "mp" },
             { "empty" },
+            { "text", "integer" },
+            { "repeat" },
         }
     }
 }
@@ -629,7 +716,7 @@ implement {
 
 implement {
     name      = "defineshift",
-    actions   = { nodes.shifts.define, context },
+    actions   = { shifts.define, context },
     arguments = {
         {
             { "continue" },
@@ -643,12 +730,12 @@ implement {
 implement {
     name     = "enableshifts",
     onlyonce = true,
-    actions  = nodes.shifts.enable
+    actions  = shifts.enable
 }
 
 implement {
     name      = "definelinefiller",
-    actions   = { nodes.linefillers.define, context },
+    actions   = { linefillers.define, context },
     arguments = {
         {
             { "method", "integer" },
@@ -670,5 +757,40 @@ implement {
 implement {
     name     = "enablelinefillers",
     onlyonce = true,
-    actions  = nodes.linefillers.enable
+    actions  = linefillers.enable
+}
+
+-- We add a bonus feature here (experiment):
+
+interfaces.implement {
+    name      = "autorule",
+    arguments = {
+        {
+            { "width", "dimension" },
+            { "height", "dimension" },
+            { "depth", "dimension" },
+            { "xoffset", "dimension" },
+            { "yoffset", "dimension" },
+            { "left", "dimension" },
+            { "right", "dimension" },
+        },
+    },
+    actions   = function(t)
+        local n = new_rule(
+            t.width or running,
+            t.height or running,
+            t.depth or running
+        )
+        setattrlist(n,true)
+        setoffsets(n,t.xoffset,t.yoffset) -- ,t.left, t.right
+        local l = t.left
+        local r = t.right
+        if l then
+            setfield(n,"left",l)
+        end
+        if r then
+            setfield(n,"right",r)
+        end
+        context(tonode(n))
+    end
 }
