@@ -684,8 +684,9 @@ end
 lmx.make = lmxmake
 
 function lmx.show(name,variables)
+    -- todo: pcall
     local htmfile = lmxmake(name,variables)
-    lmx.popupfile(htmfile)
+    -- lmx.popupfile(htmfile)
     return htmfile
 end
 
@@ -695,6 +696,8 @@ if arg then
     if     arg[1] == "--show"    then if arg[2] then lmx.show   (arg[2])                        end
     elseif arg[1] == "--convert" then if arg[2] then lmx.convert(arg[2], arg[3] or "temp.html") end
     end
+else
+    return lmx
 end
 
 -- Test 1:
