@@ -1,6 +1,6 @@
 if not modules then modules = { } end modules ['m-asymptote'] = {
     version   = 1.001,
-    comment   = "companion to m-pstricks.mkiv",
+    comment   = "companion to m-asymptote.mkiv",
     author    = "Hans Hagen, PRAGMA-ADE, Hasselt NL",
     copyright = "PRAGMA ADE / ConTeXt Development Team",
     license   = "see context related readme files"
@@ -17,7 +17,7 @@ sandbox.registerrunner {
     name     = "asymptote prc",
     program  = "asy",
     method   = "execute",
-    template = '-noV -config="" -tex=context -outformat="prc" "%filename%"',
+    template = [[-noV -config="" -tex=context -outformat="prc" %filename%]],
     checkers = { filename = "readable" },
 }
 
@@ -25,7 +25,7 @@ sandbox.registerrunner {
     name     = "asymptote pdf",
     program  = "asy",
     method   = "execute",
-    template = '-noV -config="" -tex=context -outformat="pdf" "%filename%"',
+    template = [[-noV -config="" -tex=context -outformat="pdf" %filename%]],
     checkers = { filename = "readable" },
 }
 
